@@ -226,6 +226,39 @@ ggit_config_find_system (void)
 	return path;
 }
 
+/**
+ * ggit_config_open_level:
+ * @config: a #GgitConfig.
+ * @level: the level to open.
+ * @error: a #GError for error reporting, or %NULL.
+ *
+ * Open a specific level config derived from a multi-level one.
+ *
+ * Returns: (transfer full): the configuration at @level, or %NULL.
+ *
+ **/
+GgitConfig *
+ggit_config_open_level (GgitConfig       *config,
+                        GgitConfigLevel   level,
+                        GError          **error)
+{
+	gint ret;
+	git_config *out;
+
+	g_return_val_if_fail (GGIT_IS_CONFIG (config), NULL);
+	g_return_val_if_fail (error == NULL || *error == NULL, NULL);
+
+	ret = git_config_open_level (&out, _ggit_native_get (config), (git_config_level_t)level);
+
+	if (ret != GIT_OK)
+	{
+		_ggit_error_set (error, ret);
+		return NULL;
+	}
+
+	return _ggit_config_wrap (out);
+}
+
 /**
  * ggit_config_add_file:
  * @config: a #GgitConfig.
@@ -262,6 +295,9 @@ ggit_config_add_file (GgitConfig      *config,
 	ret = git_config_add_file_ondisk (_ggit_native_get (config),
 	                                  path,
 	                                  (git_config_level_t)level,
+#if LIBGIT2_SOVERSION >= 27
+	                                  NULL,
+#endif
 	                                  force);
 	g_free (path);
 
@@ -556,6 +592,41 @@ ggit_config_set_string (GgitConfig   *config,
 	return TRUE;
 }
 
+/**
+ * ggit_config_get_entry:
+ * @config: a #GgitConfig.
+ * @name: the configuration name.
+ * @error: a #GError for error reporting, or %NULL.
+ *
+ * Get #GgitConfigEntry of a config variable.
+ *
+ * Returns: (transfer full): the entry of @name, or %NULL if such a value
+ *                           does not exist.
+ *
+ **/
+GgitConfigEntry *
+ggit_config_get_entry (GgitConfig   *config,
+                       const gchar  *name,
+                       GError      **error)
+{
+	git_config_entry *entry;
+	gint ret;
+
+	g_return_val_if_fail (GGIT_IS_CONFIG (config), NULL);
+	g_return_val_if_fail (name != NULL, NULL);
+	g_return_val_if_fail (error == NULL || *error == NULL, NULL);
+
+	ret = git_config_get_entry (&entry, _ggit_native_get (config), name);
+
+	if (ret != GIT_OK)
+	{
+		_ggit_error_set (error, ret);
+		return NULL;
+	}
+
+	return _ggit_config_entry_wrap (entry);
+}
+
 /**
  * ggit_config_delete_entry:
  * @config: a #GgitConfig.
