@@ -230,11 +230,11 @@ ggit_diff_format_email_options_init (GgitDiffFormatEmailOptions *options)
  *
  * Create a new checkout options object.
  *
- * Returns: a #GgitDiffFormatEmailOptions.
+ * Returns: (transfer full) (nullable): a #GgitDiffFormatEmailOptions or %NULL.
  *
  **/
 GgitDiffFormatEmailOptions *
-ggit_diff_format_email_options_new ()
+ggit_diff_format_email_options_new (void)
 {
 	return g_object_new (GGIT_TYPE_DIFF_FORMAT_EMAIL_OPTIONS, NULL);
 }
@@ -391,7 +391,7 @@ ggit_diff_format_email_options_set_total_patches (GgitDiffFormatEmailOptions *op
  *
  * Get the object id.
  *
- * Returns: (transfer none): a #GgitOId.
+ * Returns: (transfer none) (nullable): a #GgitOId or %NULL.
  *
  **/
 GgitOId *
@@ -446,7 +446,7 @@ ggit_diff_format_email_options_set_id (GgitDiffFormatEmailOptions *options,
  *
  * Get the summary.
  *
- * Returns: the summary.
+ * Returns: (transfer none) (nullable): the summary.
  *
  **/
 const gchar *
@@ -492,7 +492,7 @@ ggit_diff_format_email_options_set_summary (GgitDiffFormatEmailOptions *options,
  *
  * Get the author.
  *
- * Returns: (transfer none): a #GgitSignature.
+ * Returns: (transfer none) (nullable): a #GgitSignature or %NULL.
  *
  **/
 GgitSignature *
@@ -536,7 +536,7 @@ ggit_diff_format_email_options_set_author (GgitDiffFormatEmailOptions *options,
 	if (author)
 	{
 		priv->author = ggit_signature_copy (author);
-		priv->options.author = _ggit_native_get (author);
+		priv->options.author = _ggit_native_get (priv->author);
 	}
 
 	g_object_notify (G_OBJECT (options), "author");
