@@ -71,23 +71,60 @@ enum
 	PROP_REPOSITORY
 };
 
+static void
+wrapper_data_init (CallbackWrapperData *data)
+{
+	if (data != NULL)
+	{
+		data->cached_deltas =
+			g_hash_table_new_full (g_str_hash,
+			                       g_str_equal,
+			                       g_free,
+			                       (GDestroyNotify) ggit_diff_delta_unref);
+
+		data->cached_hunks =
+			g_hash_table_new_full (g_str_hash,
+			                       g_str_equal,
+			                       g_free,
+			                       (GDestroyNotify) ggit_diff_hunk_unref);
+	}
+}
+
 static GgitDiffDelta *
 wrap_diff_delta_cached (CallbackWrapperData  *data,
                         const git_diff_delta *delta)
 {
 	GgitDiffDelta *gdelta;
+	gchar *key;
 
 	if (!delta)
 	{
 		return NULL;
 	}
 
-	gdelta = g_hash_table_lookup (data->cached_deltas, delta);
+	if (delta->old_file.path != NULL)
+	{
+		key = g_strdup_printf ("%s%s",
+		                       delta->old_file.path,
+		                       git_oid_tostr_s (&delta->old_file.id));
+	}
+	else
+	{
+		key = g_strdup_printf ("%s%s",
+		                       delta->new_file.path,
+		                       git_oid_tostr_s (&delta->new_file.id));
+	}
+
+	gdelta = g_hash_table_lookup (data->cached_deltas, key);
 
 	if (!gdelta)
 	{
 		gdelta = _ggit_diff_delta_wrap (delta);
-		g_hash_table_insert (data->cached_deltas, (gpointer) delta, gdelta);
+		g_hash_table_insert (data->cached_deltas, key, gdelta);
+	}
+	else
+	{
+		g_free (key);
 	}
 
 	return gdelta;
@@ -95,21 +132,42 @@ wrap_diff_delta_cached (CallbackWrapperData  *data,
 
 static GgitDiffHunk *
 wrap_diff_hunk_cached (CallbackWrapperData *data,
+                       const git_diff_delta *delta,
                        const git_diff_hunk *hunk)
 {
 	GgitDiffHunk *ghunk;
+	gchar *key;
 
-	if (!hunk)
+	if (!delta || !hunk)
 	{
 		return NULL;
 	}
 
-	ghunk = g_hash_table_lookup (data->cached_hunks, hunk);
+	if (delta->old_file.path != NULL)
+	{
+		key = g_strdup_printf ("%s%s%s",
+		                       delta->old_file.path,
+		                       git_oid_tostr_s (&delta->old_file.id),
+		                       hunk->header);
+	}
+	else
+	{
+		key = g_strdup_printf ("%s%s%s",
+		                       delta->new_file.path,
+		                       git_oid_tostr_s (&delta->new_file.id),
+		                       hunk->header);
+	}
+
+	ghunk = g_hash_table_lookup (data->cached_hunks, key);
 
 	if (!ghunk)
 	{
 		ghunk = _ggit_diff_hunk_wrap (hunk);
-		g_hash_table_insert (data->cached_hunks, (gpointer) hunk, ghunk);
+		g_hash_table_insert (data->cached_hunks, g_strdup (key), ghunk);
+	}
+	else
+	{
+		g_free (key);
 	}
 
 	return ghunk;
@@ -195,7 +253,7 @@ ggit_diff_hunk_callback_wrapper (const git_diff_delta *delta,
 	gint ret;
 
 	gdelta = wrap_diff_delta_cached (data, delta);
-	ghunk = wrap_diff_hunk_cached (data, hunk);
+	ghunk = wrap_diff_hunk_cached (data, delta, hunk);
 
 	ret = data->hunk_cb (gdelta, ghunk, data->user_data);
 
@@ -228,7 +286,7 @@ ggit_diff_line_callback_wrapper (const git_diff_delta *delta,
 	}
 
 	gdelta = wrap_diff_delta_cached (data, delta);
-	ghunk = wrap_diff_hunk_cached (data, hunk);
+	ghunk = wrap_diff_hunk_cached (data, delta, hunk);
 
 	gline = line == NULL ? NULL : _ggit_diff_line_wrap (line, encoding);
 
@@ -380,7 +438,8 @@ _ggit_diff_wrap (GgitRepository *repository,
  * If @diff_options is %NULL then the defaults specified in
  * ggit_diff_options_new() are used.
  *
- * Returns: a newly allocated #GgitDiff if there was no error, %NULL otherwise.
+ * Returns: (transfer full) (nullable): a newly allocated #GgitDiff if
+ * there was no error, %NULL otherwise.
  */
 GgitDiff *
 ggit_diff_new_tree_to_tree (GgitRepository   *repository,
@@ -427,7 +486,8 @@ ggit_diff_new_tree_to_tree (GgitRepository   *repository,
  * If @diff_options is %NULL then the defaults specified in
  * ggit_diff_options_new() are used.
  *
- * Returns: a newly allocated #GgitDiff if there was no error, %NULL otherwise.
+ * Returns: (transfer full) (nullable): a newly allocated #GgitDiff if
+ * there was no error, %NULL otherwise.
  */
 GgitDiff *
 ggit_diff_new_tree_to_index (GgitRepository   *repository,
@@ -472,7 +532,8 @@ ggit_diff_new_tree_to_index (GgitRepository   *repository,
  * If @diff_options is %NULL then the defaults specified in
  * ggit_diff_options_new() are used.
  *
- * Returns: a newly allocated #GgitDiff if there was no error, %NULL otherwise.
+ * Returns: (transfer full) (nullable): a newly allocated #GgitDiff if
+ * there was no error, %NULL otherwise.
  */
 GgitDiff *
 ggit_diff_new_index_to_workdir (GgitRepository   *repository,
@@ -513,7 +574,8 @@ ggit_diff_new_index_to_workdir (GgitRepository   *repository,
  * If @diff_options is %NULL then the defaults specified in
  * ggit_diff_options_new() are used.
  *
- * Returns: a newly allocated #GgitDiff if there was no error, %NULL otherwise.
+ * Returns: (transfer full) (nullable): a newly allocated #GgitDiff if
+ * there was no error, %NULL otherwise.
  */
 GgitDiff *
 ggit_diff_new_tree_to_workdir (GgitRepository   *repository,
@@ -606,11 +668,11 @@ ggit_diff_foreach (GgitDiff              *diff,
 	g_return_if_fail (file_cb != NULL && binary_cb != NULL && hunk_cb != NULL && line_cb != NULL);
 	g_return_if_fail (error == NULL || *error == NULL);
 
+	wrapper_data_init (&wrapper_data);
+
 	wrapper_data.user_data = user_data;
 	wrapper_data.diff = diff;
 
-	wrapper_data.cached_deltas = g_hash_table_new_full (g_direct_hash, g_direct_equal, NULL, (GDestroyNotify) ggit_diff_delta_unref);
-	wrapper_data.cached_hunks = g_hash_table_new_full (g_direct_hash, g_direct_equal, NULL, (GDestroyNotify) ggit_diff_hunk_unref);
 
 	if (file_cb != NULL)
 	{
@@ -674,14 +736,13 @@ ggit_diff_print (GgitDiff              *diff,
 	g_return_if_fail (print_cb != NULL);
 	g_return_if_fail (error == NULL || *error == NULL);
 
+	wrapper_data_init (&wrapper_data);
+
 	wrapper_data.user_data = user_data;
 	wrapper_data.diff = diff;
 
 	wrapper_data.line_cb = print_cb;
 
-	wrapper_data.cached_deltas = g_hash_table_new_full (g_direct_hash, g_direct_equal, NULL, (GDestroyNotify) ggit_diff_delta_unref);
-	wrapper_data.cached_hunks = g_hash_table_new_full (g_direct_hash, g_direct_equal, NULL, (GDestroyNotify) ggit_diff_hunk_unref);
-
 	ret = git_diff_print (_ggit_native_get (diff), (git_diff_format_t)type,
 	                      ggit_diff_line_callback_wrapper,
 	                      &wrapper_data);
@@ -703,7 +764,7 @@ ggit_diff_print (GgitDiff              *diff,
  *
  * Create an e-mail ready patch from a diff.
  *
- * Returns: (transfer full): the patch or %NULL if an error occurred.
+ * Returns: (transfer full) (nullable): the patch or %NULL if an error occurred.
  *
  **/
 gchar *
@@ -760,7 +821,7 @@ ggit_diff_get_num_deltas (GgitDiff *diff)
  *
  * Get the delta at the specified index.
  *
- * Returns: (transfer full): a #GgitDiffDelta.
+ * Returns: (transfer full) (nullable): a #GgitDiffDelta or %NULL.
  *
  **/
 GgitDiffDelta *
@@ -827,6 +888,8 @@ ggit_diff_blobs (GgitBlob              *old_blob,
 
 	gdiff_options = _ggit_diff_options_get_diff_options (diff_options);
 
+	wrapper_data_init (&wrapper_data);
+
 	wrapper_data.user_data = user_data;
 
 	if (file_cb != NULL)
@@ -853,9 +916,6 @@ ggit_diff_blobs (GgitBlob              *old_blob,
 		wrapper_data.line_cb = line_cb;
 	}
 
-	wrapper_data.cached_deltas = g_hash_table_new_full (g_direct_hash, g_direct_equal, NULL, (GDestroyNotify) ggit_diff_delta_unref);
-	wrapper_data.cached_hunks = g_hash_table_new_full (g_direct_hash, g_direct_equal, NULL, (GDestroyNotify) ggit_diff_hunk_unref);
-
 	ret = git_diff_blobs (old_blob ? _ggit_native_get (old_blob) : NULL,
 	                      old_as_path,
 	                      new_blob ? _ggit_native_get (new_blob) : NULL,
@@ -921,6 +981,8 @@ ggit_diff_blob_to_buffer (GgitBlob              *old_blob,
 
 	gdiff_options = _ggit_diff_options_get_diff_options (diff_options);
 
+	wrapper_data_init (&wrapper_data);
+
 	wrapper_data.user_data = user_data;
 
 	if (buffer_len == -1)
@@ -952,9 +1014,6 @@ ggit_diff_blob_to_buffer (GgitBlob              *old_blob,
 		wrapper_data.line_cb = line_cb;
 	}
 
-	wrapper_data.cached_deltas = g_hash_table_new_full (g_direct_hash, g_direct_equal, NULL, (GDestroyNotify) ggit_diff_delta_unref);
-	wrapper_data.cached_hunks = g_hash_table_new_full (g_direct_hash, g_direct_equal, NULL, (GDestroyNotify) ggit_diff_hunk_unref);
-
 	ret = git_diff_blob_to_buffer (old_blob ? _ggit_native_get (old_blob) : NULL,
 	                               old_as_path,
 	                               (const gchar *) buffer,
